% 'Matchine' software by Joan Alexis Glaunes, Copyright (C) Universite Paris Descartes, All Rights Reserved 2005-2006, version 1.0
%
% Any use of this code should make reference to at least one of the following references:
%
% - [for the use of 'measure matching']:
% J. Glaunes, L. Younes and A. Trouve, Diffeomorphic matching of distributions: A new approach for unlabelled point-sets and sub-manifolds matching, Proc. of the 2004 IEEE Computer Society Conference on Computer Vision and Pattern Recognition. (CVPR'04), Vol. 2, pp. 712--718, DOI: 10.1109/CVPR.2004.81
%
% - [for the use of 'curve matching']:
% J. Glaunes, A. Qiu, M. Miller, L. Younes, Large Deformation Diffeomorphic Metric Curve Mapping, International Journal of Computer vision, Springer, 2008, Vol. 80, No. 3, pp. 317--336, DOI: 10.1007/s11263-008-0141-9
%
% - [for the use of 'surface matching']:
% M. Vaillant and J. Glaunes, Surface Matching via Currents, Proc. of Information Processing in Medical Imaging (IPMI'05), Lecture Notes in Computer Science vol. 3565, Springer 2005, pp. 381--392



function s = landmarks(s)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Landmarks matching - to be used with match.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% contact : http://www.mi.parisdescartes.fr/~glaunes/

method = 'landmarks';

% y (target points) is required
if ~isfield(s,'y')
    error('input structure must contain field <y> (target points)');
end

y = s.y;

% y must be 3D data
if size(y,1)~=3
    error('field <y> must be 3-rows matrix');
end

ny = size(y,2);
if ~isfield(s,'vx')
    vx = 1:ny;
else
    vx = s.vx;
end

matching = @funmatching;
gradmatching = @fungradmatching;

clear s
savestruct('s',who);

vx = 3*vx(:)';
vx = [vx-2;vx-1;vx];
vx = vx(:);

y = y(:);

    function R = funmatching(phix)
        R = sum((phix(vx)-y).^2);
    end

    function g = fungradmatching(phix)    
        g = zeros(size(phix));
        g(vx) = 2*(phix(vx)-y);
    end

end

